@@ -18,22 +18,31 @@ use MIME::Base64 qw(encode_base64 decode_base64);
 use POSIX qw(strftime);
 use Data::Dumper qw(Dumper);
 use URI::Escape qw(uri_escape_utf8);
+use Moose::Util::TypeConstraints;
 
 use Net::Amazon::EMR::Coercions;
 
+use Net::Amazon::EMR::Application;
 use Net::Amazon::EMR::AddInstanceGroupsResult;
+use Net::Amazon::EMR::AddJobFlowStepsResult;
 use Net::Amazon::EMR::BootstrapActionConfig;
 use Net::Amazon::EMR::BootstrapActionDetail;
+use Net::Amazon::EMR::DescribeClusterResult;
 use Net::Amazon::EMR::DescribeJobFlowsResult;
 use Net::Amazon::EMR::HadoopJarStepConfig;
-use Net::Amazon::EMR::InstanceGroupDetail;
 use Net::Amazon::EMR::InstanceGroupConfig;
+use Net::Amazon::EMR::InstanceGroupDetail;
 use Net::Amazon::EMR::InstanceGroupModifyConfig;
 use Net::Amazon::EMR::JobFlowDetail;
 use Net::Amazon::EMR::JobFlowExecutionStatusDetail;
 use Net::Amazon::EMR::JobFlowInstancesConfig;
 use Net::Amazon::EMR::JobFlowInstancesDetail;
 use Net::Amazon::EMR::KeyValue;
+use Net::Amazon::EMR::ListBootstrapActionsResult;
+use Net::Amazon::EMR::ListClustersResult;
+use Net::Amazon::EMR::ListInstanceGroupsResult;
+use Net::Amazon::EMR::ListInstancesResult;
+use Net::Amazon::EMR::ListStepsResult;
 use Net::Amazon::EMR::PlacementType;
 use Net::Amazon::EMR::RunJobFlowResult;
 use Net::Amazon::EMR::ScriptBootstrapActionConfig;
@@ -42,7 +51,7 @@ use Net::Amazon::EMR::StepDetail;
 use Net::Amazon::EMR::StepExecutionStatusDetail;
 use Net::Amazon::EMR::SupportedProductConfig;
 
-our $VERSION = '0.19';
+our $VERSION = '0.20';
 
 has 'AWSAccessKeyId'    => ( is => 'ro', isa => 'Str', required => 1 );
 has 'SecretAccessKey'   => ( is => 'ro', isa => 'Str', required => 1 );
@@ -58,6 +67,11 @@ has 'base_url'                  => (
 );
 has 'max_failures'      => ( is => 'ro', isa => 'Int', default => 5 );
 
+subtype 'Net::Amazon::EMR::Type::ListClusters::ClusterState' => as 'Str' => where { m/^(?:STARTING | BOOTSTRAPPING | RUNNING | WAITING | TERMINATING | TERMINATED | TERMINATED_WITH_ERRORS)$/x };
+subtype 'Net::Amazon::EMR::Type::ListInstances::InstanceGroupTypes' => as 'Str' => where { m/^(?:MASTER | CORE | TASK)$/x };
+subtype 'Net::Amazon::EMR::Type::ListSteps::StepStates' => as 'Str' => where { m/^(?:PENDING | RUNNING | COMPLETED | CANCELLED | FAILED | INTERRUPTED)$/x };
+
+
 after 'BUILDARGS' => sub {
     unless (Log::Log4perl->initialized) {
         Log::Log4perl->easy_init($ERROR);
@@ -150,7 +164,8 @@ sub _sign {
     }
 
     my $ref = $xs->XMLin($xml);
-
+#    print $xml . "\n";
+#    use Data::Dumper; print Dumper($ref);
     $self->log->debug(sub { "Action $action result: " . Dumper($ref) });
 
     if ($ref->{Error}) {
@@ -231,9 +246,37 @@ sub add_job_flow_steps {
 
     my $xml = $self->_sign(Action  => 'AddJobFlowSteps', %flat_args);
 
+    return Net::Amazon::EMR::AddJobFlowStepsResult->new($xml);
+}
+
+sub add_tags {
+    my ($self, %args) = validated_hash( \@_, 
+        ResourceId => { isa => 'Str' },
+        Tags => { isa =>  'Net::Amazon::EMR::Type::ArrayRefofTag', 
+                  coerce => 1},
+                         );
+
+    my %flat_args;
+    _to_flat_args(\%args, \%flat_args);
+
+    my $xml = $self->_sign(Action  => 'AddTags', %flat_args);
+
     return 1;
 }
 
+sub describe_cluster {
+    my ($self, %args) = validated_hash( \@_, 
+        ClusterId => { isa => 'Str' },
+                                    );
+
+    my %flat_args;
+    _to_flat_args(\%args, \%flat_args);
+
+    my $xml = $self->_sign(Action  => 'DescribeCluster', %flat_args);
+
+    return Net::Amazon::EMR::DescribeClusterResult->new($xml);
+}
+
 sub describe_job_flows {
     my ($self, %args) = validated_hash( \@_, 
         CreatedAfter    => { isa => 'Net::Amazon::EMR::Type::DateTime', 
@@ -253,6 +296,116 @@ sub describe_job_flows {
     return Net::Amazon::EMR::DescribeJobFlowsResult->new($xml);
 }
 
+sub describe_steps {
+    my ($self, %args) = validated_hash( \@_, 
+        ClusterId => { isa => 'Str' },
+        StepId => { isa => 'Str' },
+                                    );
+
+    my %flat_args;
+    _to_flat_args(\%args, \%flat_args);
+
+    my $xml = $self->_sign(Action  => 'DescribeStep', %flat_args);
+
+    return Net::Amazon::EMR::DescribeStepResult->new($xml);
+}
+
+sub list_bootstrap_actions {
+    my ($self, %args) = validated_hash( \@_, 
+                                        ClusterId => { isa => 'Str' },
+                                        Marker => { isa => 'Str', optional => 1 },
+                                    );
+
+    my %flat_args;
+    _to_flat_args(\%args, \%flat_args);
+
+    my $xml = $self->_sign(Action  => 'ListBootstrapActions', %flat_args);
+
+    return Net::Amazon::EMR::ListBootstrapActionsResult->new($xml);
+}
+
+sub list_clusters {
+    my ($self, %args) = validated_hash( \@_, 
+                                        ClusterStates => {
+                                            isa => 'Net::Amazon::EMR::Type::ListClusters::ClusterState',
+                                            coerce => 1,
+                                            optional => 1,
+                                        },
+                                        CreatedAfter    => { isa => 'Net::Amazon::EMR::Type::DateTime', 
+                                                             optional => 1, 
+                                                             coerce => 1 },
+                                        CreatedBefore   => { isa => 'Net::Amazon::EMR::Type::DateTime', 
+                                                             optional => 1, 
+                                                             coerce => 1 },
+                                        Marker => { isa => 'Str', optional => 1 },
+                                    );
+
+    my %flat_args;
+    _to_flat_args(\%args, \%flat_args);
+
+    my $xml = $self->_sign(Action  => 'ListClusters', %flat_args);
+
+    return Net::Amazon::EMR::ListClustersResult->new($xml);
+}
+
+sub list_instance_groups {
+    my ($self, %args) = validated_hash( \@_, 
+                                        ClusterId => { isa => 'Str' },
+                                        Marker => { isa => 'Str', optional => 1 },
+                                    );
+
+    my %flat_args;
+    _to_flat_args(\%args, \%flat_args);
+
+    my $xml = $self->_sign(Action  => 'ListInstanceGroups', %flat_args);
+
+    return Net::Amazon::EMR::ListInstanceGroupsResult->new($xml);
+}
+
+sub list_instances {
+    my ($self, %args) = validated_hash( \@_, 
+                                        ClusterId => { isa => 'Str' },
+                                        InstanceGroupId => { isa => 'Str', optional => 1 },
+                                        InstanceGroupTypes => {
+                                            isa => 'ArrayRef[Net::Amazon::EMR::Type::ListInstances::InstanceGroupTypes]',
+                                            coerce => 1,
+                                            optional => 1,
+                                        },
+
+                                        Marker => { isa => 'Str', optional => 1 },
+                                    );
+
+    my %flat_args;
+    _to_flat_args(\%args, \%flat_args);
+
+    my $xml = $self->_sign(Action  => 'ListInstances', %flat_args);
+
+    return Net::Amazon::EMR::ListInstancesResult->new($xml);
+}
+
+sub list_steps {
+    my ($self, %args) = validated_hash( \@_, 
+                                        ClusterId => { isa => 'Str' },
+                                        Marker => { isa => 'Str', optional => 1 },
+                                        StepIds => { isa => 'ArrayRef[Net::Amazon::EMR::Type::LongConstrainedStr]',
+                                                     coerce => 1,
+                                                     optional => 1 },
+                                        StepStates => {
+                                            isa => 'ArrayRef[Net::Amazon::EMR::Type::ListSteps::StepStates]',
+                                            coerce => 1,
+                                            optional => 1,
+                                        },
+
+                                    );
+
+    my %flat_args;
+    _to_flat_args(\%args, \%flat_args);
+
+    my $xml = $self->_sign(Action  => 'ListInstances', %flat_args);
+
+    return Net::Amazon::EMR::ListStepsResult->new($xml);
+}
+
 sub modify_instance_groups {
     my ($self, %args) = validated_hash( \@_, 
                                         InstanceGroups => { isa => 'Net::Amazon::EMR::Type::ArrayRefofInstanceGroupModifyConfig', 
@@ -269,26 +422,69 @@ sub modify_instance_groups {
 
 }
 
-sub run_job_flow {
+sub remove_tags {
     my ($self, %args) = validated_hash( \@_, 
-        AdditionalInfo  => { isa => 'Str', optional => 1 },
-        AmiVersion      => { isa => 'Str', optional => 1 },
+                                        ResourceId => { isa => 'Str' },
+                                        TagKeyss => { isa =>  'Net::Amazon::EMR::Type::ArrayRefofStr', 
+                                                      coerce => 1},
+                                    );
+
+    my %flat_args;
+    _to_flat_args(\%args, \%flat_args);
+
+    my $xml = $self->_sign(Action  => 'RemoveTags', %flat_args);
+
+    return 1;
+}
+
+sub run_job_flow {
+    my ($self, %args) = validated_hash(
+        \@_,
+        AdditionalInfo  => { isa => 'Net::Amazon::EMR::Type::LongConstrainedStr | Undef',
+                             coerce => 1,
+                             optional => 1 },
+        AmiVersion      => { isa => 'Net::Amazon::EMR::Type::ConstrainedStr | Undef',
+                             coerce => 1,
+                             optional => 1 },
+        Applications    => { isa => 'Net::Amazon::EMR::Type::ArrayRefofApplication | Undef',
+                             coerce => 1,
+                             optional => 1 },
         BootstrapActions => { isa => 'Net::Amazon::EMR::Type::ArrayRefofBootstrapActionConfig | Undef', 
                               optional => 1, 
                               coerce => 1 },
+        Configurations   => { isa => 'Net::Amazon::EMR::Type::ArrayRefofConfiguration | Undef',
+                              coerce => 1,
+                              optional => 1 },
         Instances       => { isa => 'Net::Amazon::EMR::Type::JobFlowInstancesConfig', coerce => 1 },
-        JobFlowRole  => { isa => 'Str | Undef', optional => 1 },
-        LogUri  => { isa => 'Str | Undef', optional => 1 },
-        Name    => { isa => 'Str' },
+        JobFlowRole  => { isa => 'Net::Amazon::EMR::Type::LongConstrainedStr | Undef',
+                          coerce => 1,
+                          optional => 1 },
+        LogUri  => { isa => 'Net::Amazon::EMR::Type::LongConstrainedStr | Undef',
+                     coerce => 1,
+                     optional => 1 },
+        Name    => { isa => 'Net::Amazon::EMR::Type::ConstrainedStr',
+                     coerce => 1 },
         NewSupportedProducts => { isa => 'Net::Amazon::EMR::Type::ArrayRefofSupportedProductConfig | Undef', 
-                              optional => 1, 
-                              coerce => 1 },
-        ServiceRole  => { isa => 'Str | Undef', optional => 1 },
-        Steps   => { isa => 'Net::Amazon::EMR::Type::ArrayRefofStepConfig', optional => 1, coerce => 1 },
-        SupportedProducts => { isa => 'ArrayRef[Str] | Undef', optional => 1 },
-        Tags => { isa => 'ArrayRef[Str] | Undef', optional => 1 },
-        VisibleToAllUsers => { isa => 'Net::Amazon::EMR::Type::Bool', optional => 1 },
-                                        );
+                                  optional => 1, 
+                                  coerce => 1 },
+        ReleaseLabel  => { isa => 'Net::Amazon::EMR::Type::ConstrainedStr | Undef',
+                           coerce => 1,
+                           optional => 1 },
+        ServiceRole  => { isa => 'Net::Amazon::EMR::Type::LongConstrainedStr | Undef',
+                          coerce => 1,
+                          optional => 1 },
+        Steps   => { isa => 'Net::Amazon::EMR::Type::ArrayRefofStepConfig',
+                     coerce => 1,
+                     optional => 1 },
+        SupportedProducts => { isa => 'ArrayRef[Net::Amazon::EMR::Type::ConstrainedStr] | Undef',
+                               optional => 1 },
+        Tags => { isa => 'Net::Amazon::EMR::Type::ArrayRefofTag | Undef',
+                  coerce => 1,
+                  optional => 1 },
+        VisibleToAllUsers => { isa => 'Net::Amazon::EMR::Type::Bool',
+                               coerce => 1,
+                               optional => 1 },
+    );
 
     _convert_bool(\%args, 'VisibleToAllUsers');
 
@@ -302,11 +498,13 @@ sub run_job_flow {
 
 
 sub set_termination_protection {
-    my ($self, %args) = validated_hash( \@_, 
-                                        JobFlowIds      => { isa => 'ArrayRef' },
-                                        TerminationProtected => { isa => 'Net::Amazon::EMR::Type::Bool', 
-                                                                  coerce => 1 },
-        );
+    my ($self, %args) = validated_hash(
+        \@_, 
+        JobFlowIds      => { isa => 'ArrayRef[Net::Amazon::EMR::Type::LongConstrainedStr]',
+                             coerce => 1 },
+        TerminationProtected => { isa => 'Net::Amazon::EMR::Type::Bool', 
+                                  coerce => 1 },
+    );
     _convert_bool(\%args, 'TerminationProtected');
 
     my %flat_args;
@@ -318,7 +516,8 @@ sub set_termination_protection {
 
 sub set_visible_to_all_users {
     my ($self, %args) = validated_hash( \@_, 
-        JobFlowIds      => { isa => 'ArrayRef' },
+        JobFlowIds      => { isa => 'ArrayRef[Net::Amazon::EMR::Type::LongConstrainedStr]',
+                             coerce => 1 },
         VisibleToAllUsers => { isa => 'Net::Amazon::EMR::Type::Bool', 
                                coerce => 1 },
         );
@@ -333,7 +532,8 @@ sub set_visible_to_all_users {
 
 sub terminate_job_flows {
     my ($self, %args) = validated_hash( \@_, 
-        JobFlowIds      => { isa => 'ArrayRef' },
+        JobFlowIds      => { isa => 'ArrayRef[Net::Amazon::EMR::Type::LongConstrainedStr]',
+                             coerce => 1 },
                          );
     my %flat_args;
     _to_flat_args(\%args, \%flat_args);
@@ -503,17 +703,52 @@ AddInstanceGroups adds an instance group to a running cluster. Returns a L<Net::
 
 =head2 add_job_flow_steps(%params)
 
-AddJobFlowSteps adds new steps to a running job flow. Returns 1 on success.
+AddJobFlowSteps adds new steps to a running job flow. Returns a L<Net::Amazon::EMR::AddJobFlowStepsResult> object.
 
+=head2 add_tags(%params)
+
+Adds tags to an Amazon EMR resource. Returns 1 on success.
+
+=head2 describe_cluster(%params)
+
+Returns a L<Net::Amazon::EMR::DescribeClustersResult> that provides cluster-level details including status, hardware and software configuration, VPC settings, etc.
 
 =head2 describe_job_flows(%params)
 
 Returns a L<Net::Amazon::EMR::RunJobFlowResult> that describes the job flows that match all of the supplied parameters.
 
+=head2 describe_step(%params)
+
+Returns a L<Net::Amazon::EMR::DescribeStepResult> that provides detail about the cluster step.
+
+=head2 list_bootstrap_actions(%params)
+
+Returns a L<Net::Amazon::EMR::ListBootstrapActionsResult> that provides detail about the bootstrap actions associated with the cluster.
+
+=head2 list_clusters(%params)
+
+Returns a L<Net::Amazon::EMR::ListClustersResult> that provides the status of all clusters visible to this AWS account.
+
+=head2 list_instance_groups(%params)
+
+Returns a L<Net::Amazon::EMR::ListInstanceGroupsResult> that provides all available details about the instance groups in a cluster.
+
+=head2 list_instances(%params)
+
+Returns a L<Net::Amazon::EMR::ListInstancesResult> that provides information about the cluster instances that Amazon EMR provisions on behalf of a user when it creates the cluster.
+
+=head2 list_steps(%params)
+
+Returns a L<Net::Amazon::EMR::ListStepsResult> that provides a list of steps for the cluster.
+
 =head2 modify_instance_groups(%params)
 
 Modifies the number of nodes and configuration settings of an instance group. Returns 1 on success.
 
+=head2 remove_tags(%params)
+
+Removes tags from an Amazon EMR resource. Returns 1 on success.
+
 =head2 run_job_flow(%params)
 
 Creates and starts running a new job flow. Returns a L<Net::Amazon::EMR::RunJobFlowResult> object that contains the job flow ID.
