@@ -4,10 +4,14 @@ with 'Net::Amazon::EMR::Role::AttrHash';
 use Net::Amazon::EMR::Coercions;
 
 has 'Key' => ( is => 'ro', 
-               isa => 'Str' );
+               isa => 'Net::Amazon::EMR::Type::LongConstrainedStr',
+               coerce => 1,
+           );
 
 has 'Value' => ( is => 'ro', 
-                 isa => 'Str' );
+                 isa => 'Net::Amazon::EMR::Type::LongConstrainedStr',
+                 coerce => 1,
+             );
 
 
 __PACKAGE__->meta->make_immutable();
