@@ -2,11 +2,13 @@ package Net::Amazon::EMR::StepExecutionStatusDetail;
 use Moose;
 with 'Net::Amazon::EMR::Role::AttrHash';
 use Net::Amazon::EMR::Coercions;
+use Moose::Util::TypeConstraints;
 
 
 has 'CreationDateTime' => ( is => 'ro', 
                             isa => 'Net::Amazon::EMR::Type::DateTime',
                             coerce => 1,
+                            required => 1,
     );
 
 has 'EndDateTime' => ( is => 'ro', 
@@ -15,21 +17,21 @@ has 'EndDateTime' => ( is => 'ro',
     );
 
 has 'LastStateChangeReason' => ( is => 'ro', 
-                                 isa => 'Str' );
-
-has 'ReadyDateTime' => ( is => 'ro', 
-                         isa => 'Net::Amazon::EMR::Type::DateTime',
-                         coerce => 1,
-    );
+                      isa => 'Net::Amazon::EMR::Type::StepConfig',
+                      coerce => 1,
+                  );
 
 has 'StartDateTime' => ( is => 'ro', 
                          isa => 'Net::Amazon::EMR::Type::DateTime',
                          coerce => 1,
-    );
+                     );
 
-has 'State' => ( is => 'ro', 
-                 isa => 'Str' );
+subtype 'Net::Amazon::EMR::Type::StepExecutionStatusDetail::State' => as 'Str' => where { m/^(?:PENDING | RUNNING | CONTINUE | COMPLETED | CANCELLED | FAILED | INTERRUPTED)$/x };
 
+has 'State' => ( is => 'ro',
+                 isa => 'Net::Amazon::EMR::Type::StepExecutionStatusDetail::State',
+                 required => 1,
+             );
 
 __PACKAGE__->meta->make_immutable();
 
