@@ -3,11 +3,15 @@ use Moose;
 with 'Net::Amazon::EMR::Role::AttrHash';
 use Net::Amazon::EMR::Coercions;
 
-has 'Name' => ( is => 'ro', 
-                isa => 'Str' );
-
 has 'Args' => ( is => 'ro',
-                isa => 'Str' );
+                isa => 'Net::Amazon::EMR::Type::ArrayRefofLongConstrainedStr',
+                coerce => 1,
+            );
+
+has 'Name' => ( is => 'ro', 
+                isa => 'Net::Amazon::EMR::Type::ConstrainedStr',
+                coerce => 1,
+            );
 
 __PACKAGE__->meta->make_immutable();
 
